# Generated by Django 4.2.3 on 2023-08-03 00:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0069_alter_healtheconomicshouseholdhead_hoh_employment_type_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="healtheconomicshouseholdhead",
            new_name="hoh_ethnicity_old",
            old_name="hoh_ethnicity",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicshouseholdhead",
            new_name="hoh_ethnicity_old",
            old_name="hoh_ethnicity",
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-20 21:08

import _socket
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model.validators.date
import edc_model_fields.fields.other_charfield
import edc_sites.models
import edc_utils.date
import simple_history.models
import uuid


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CommunityCareLocation",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("report_datetime", models.DateTimeField()),
                ("name", models.CharField(max_length=25, null=True, unique=True)),
                ("opening_date", models.DateField()),
                (
                    "closing_date",
                    models.DateField(
                        blank=True,
                        help_text="Complete the 'closing' date if the location is no longer being used by any group",
                        null=True,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[("open", "Open"), ("closed", "Closed")],
                        default="open",
                        max_length=25,
                    ),
                ),
                (
                    "location_type_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "gps",
                    models.CharField(
                        blank=True,
                        help_text="copy and paste directly from google maps",
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    "latitude",
                    models.FloatField(
                        blank=True,
                        help_text="in degrees. copy and paste directly from google maps",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(-90),
                            django.core.validators.MaxValueValidator(90),
                        ],
                    ),
                ),
                (
                    "longitude",
                    models.FloatField(
                        blank=True,
                        help_text="in degrees. copy and paste directly from google maps",
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(-180),
                            django.core.validators.MaxValueValidator(180),
                        ],
                    ),
                ),
                ("description", models.TextField(blank=True, null=True)),
                ("notes", models.TextField(blank=True, null=True)),
            ],
            options={
                "verbose_name": "Community Care Location",
                "verbose_name_plural": "Community Care Locations",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalPatientGroup",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "group_identifier",
                    models.CharField(db_index=True, default=uuid.uuid4, max_length=36),
                ),
                (
                    "group_identifier_as_pk",
                    models.UUIDField(db_index=True, default=uuid.uuid4, editable=False),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        validators=[edc_model.validators.date.datetime_not_future],
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="The name may be changed at anytime as long as it is unique.",
                        max_length=25,
                        verbose_name="Group name",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("New", "New"),
                            ("recruiting", "Recruiting"),
                            ("COMPLETE", "Complete/Ready"),
                            ("in_followup", "In followup"),
                            ("dissolved", "Dissolved"),
                        ],
                        default="recruiting",
                        max_length=25,
                    ),
                ),
                (
                    "ratio",
                    models.DecimalField(decimal_places=4, max_digits=10, null=True),
                ),
                (
                    "enforce_group_size_min",
                    models.BooleanField(
                        default=True, verbose_name="Enforce group size minimum"
                    ),
                ),
                (
                    "enforce_ratio",
                    models.BooleanField(
                        default=True, verbose_name="Enforce 2:1 NCD:HIV ratio"
                    ),
                ),
                (
                    "randomize_now",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        max_length=15,
                        verbose_name="Randomize now?",
                    ),
                ),
                (
                    "confirm_randomize_now",
                    models.CharField(
                        blank=True,
                        max_length=15,
                        null=True,
                        verbose_name="If YES, please confirm by typing the wors RANDOMIZE here",
                    ),
                ),
                ("randomized", models.BooleanField(blank=True, default=False)),
                ("randomized_datetime", models.DateTimeField(blank=True, null=True)),
                ("notes", models.TextField(blank=True, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Patient Group",
                "verbose_name_plural": "historical Patient Groups",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="PatientGroup",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "group_identifier",
                    models.CharField(default=uuid.uuid4, max_length=36, unique=True),
                ),
                (
                    "group_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        validators=[edc_model.validators.date.datetime_not_future],
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="The name may be changed at anytime as long as it is unique.",
                        max_length=25,
                        unique=True,
                        verbose_name="Group name",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("New", "New"),
                            ("recruiting", "Recruiting"),
                            ("COMPLETE", "Complete/Ready"),
                            ("in_followup", "In followup"),
                            ("dissolved", "Dissolved"),
                        ],
                        default="recruiting",
                        max_length=25,
                    ),
                ),
                (
                    "ratio",
                    models.DecimalField(decimal_places=4, max_digits=10, null=True),
                ),
                (
                    "enforce_group_size_min",
                    models.BooleanField(
                        default=True, verbose_name="Enforce group size minimum"
                    ),
                ),
                (
                    "enforce_ratio",
                    models.BooleanField(
                        default=True, verbose_name="Enforce 2:1 NCD:HIV ratio"
                    ),
                ),
                (
                    "randomize_now",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        max_length=15,
                        verbose_name="Randomize now?",
                    ),
                ),
                (
                    "confirm_randomize_now",
                    models.CharField(
                        blank=True,
                        max_length=15,
                        null=True,
                        verbose_name="If YES, please confirm by typing the wors RANDOMIZE here",
                    ),
                ),
                ("randomized", models.BooleanField(blank=True, default=False)),
                ("randomized_datetime", models.DateTimeField(blank=True, null=True)),
                ("notes", models.TextField(blank=True, null=True)),
            ],
            options={
                "verbose_name": "Patient Group",
                "verbose_name_plural": "Patient Groups",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="PatientGroupAppointment",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                ("appt_datetime", models.DateTimeField()),
                (
                    "appt_status",
                    models.CharField(
                        choices=[
                            ("new", "Scheduled"),
                            ("in_progress", "In Progress"),
                            ("done", "Done"),
                            ("cancelled", "Cancelled"),
                        ],
                        default="new",
                        max_length=25,
                    ),
                ),
                ("notes", models.TextField(blank=True, null=True)),
            ],
            options={
                "verbose_name": "Patient Group Appointment",
                "verbose_name_plural": "Patient Group Appointments",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.CreateModel(
            name="PatientGroupMeeting",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[("scheduled", "Scheduled"), ("Attended", "Attended")],
                        default="scheduled",
                        max_length=25,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                ("meeting_datetime", models.DateTimeField(blank=True, null=True)),
                ("notes", models.TextField(blank=True, null=True)),
                (
                    "patient_group_appointment",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="intecomm_group.patientgroupappointment",
                    ),
                ),
            ],
            options={
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
    ]



# Generated by Django 4.1.7 on 2023-05-02 22:40

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0025_patientlogreportprinthistory_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="filing_identifier",
            field=models.CharField(
                db_index=True,
                default=uuid.uuid4,
                help_text="A sequential number to label a manual patient file",
                max_length=36,
                verbose_name="File number",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="filing_identifier",
            field=models.CharField(
                default=uuid.uuid4,
                help_text="A sequential number to label a manual patient file",
                max_length=36,
                unique=True,
                verbose_name="File number",
            ),
        ),
    ]



from __future__ import annotations

from edc_model_admin.admin_site import EdcAdminSite
from edc_sites import get_current_country

from .apps import AppConfig
from .constants import UGANDA
from .intecomm_admin_site_mixin import IntecommAdminSiteMixin


class IntecommScreeningSite(IntecommAdminSiteMixin, EdcAdminSite):
    @staticmethod
    def get_hidden_models(request) -> dict:
        if get_current_country(request=request) == UGANDA:
            hidden_models = {
                "intecomm_screening": ["PatientLog", "SubjectScreening"],
            }
        else:
            hidden_models = {
                "intecomm_screening": ["PatientLogUg", "SubjectScreeningUg"],
            }
        return hidden_models


intecomm_screening_admin = IntecommScreeningSite(
    name="intecomm_screening_admin", app_label=AppConfig.name
)



# Generated by Django 4.2.4 on 2023-08-23 15:53

import _socket
import django.core.validators
import django_audit_fields.fields.hostname_modification_field
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0108_alter_concomitantmedication_device_created_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="dwelling_value_known",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="in local currency",
                max_length=25,
                verbose_name="If the owner [you] were to sell this dwelling today, do you know the approximate value?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="water_obtain_time",
            field=models.CharField(
                choices=[
                    ("on_premises", "Water on premises (includes water piped to a neighbour)"),
                    ("less_30min", "Less than 30 minutes"),
                    ("greater_30min", "30 minutes or longer"),
                    ("dont_know", "Don't know"),
                ],
                max_length=25,
                verbose_name="How long does it take to obtain drinking water?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("9", "Self employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household. If the household head has more than one job, think about their MAIN source of employment",
                max_length=25,
                verbose_name="Household head’s employment status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head’s marital status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="relationship_to_hoh",
            field=models.CharField(
                choices=[
                    ("WIFE_HUSBAND", "Wife/Husband"),
                    ("SON_DAUGHTER", "Son/Daughter"),
                    ("SON_DAUGHTERINLAW", "Son/Daughter-in-law"),
                    ("GRANDCHILD", "Grandchild"),
                    ("PARENT", "Parent"),
                    ("PARENTINLAW", "Parent-in-law"),
                    ("BROTHER_SISTER", "Brother/Sister"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="What is your relationship to the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="external_remit",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text=None,
                max_length=15,
                verbose_name="Money transfers from family members or friends residing outside the country",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="external_remit_currency",
            field=models.CharField(
                blank=True,
                choices=[
                    ("USD", "USD"),
                    ("GBP", "GBP"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don't know"),
                ],
                max_length=50,
                null=True,
                verbose_name="In what currency do you receive remittances?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="external_remit_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="financial_status",
            field=models.CharField(
                choices=[
                    ("1", "Very good"),
                    ("2", "Good"),
                    ("3", "Moderate"),
                    ("4", "Bad"),
                    ("5", "Very bad"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="Would you say your household's financial situation is?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="financial_status_compare",
            field=models.CharField(
                choices=[
                    ("1", "Among most wealthy"),
                    ("2", "Above average "),
                    ("3", "Average wealth"),
                    ("4", "Below average"),
                    ("5", "Among least wealthy"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="How would you rate your household’s financial situation compared with others in your community?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="household_debt",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="Does your household or any members of the household have current debt or outstanding loans?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="income_enough",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("Dont_know", "Don't know")],
                max_length=15,
                verbose_name="Thinking about the income for this household, do you believe that it is enough money to cover your daily living needs and obligations?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="interest",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text="(for example, from savings account or fixed deposits)?",
                max_length=15,
                verbose_name="Interest, dividends",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="interest_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="internal_remit",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text=None,
                max_length=15,
                verbose_name="Money transfers from family members or friends residing inside the country",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="internal_remit_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="more_sources",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text=None,
                max_length=15,
                verbose_name="Do you have additional sources of income not included above?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="more_sources_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="ngo_assistance",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text=None,
                max_length=15,
                verbose_name="Assistance from nongovernmental organization",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="ngo_assistance_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="pension",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text="Pensions by work",
                max_length=15,
                verbose_name="State old-age (veteran's/civil service) pension*, contributory pension fund, provident fund or social security benefit?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="pension_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="rental_income",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text=None,
                max_length=15,
                verbose_name="Income from rental of property?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="rental_income_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="selling",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text=None,
                max_length=15,
                verbose_name="Earnings from selling, trading or hawking products?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="selling_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="wages",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                help_text=None,
                max_length=15,
                verbose_name="Income from wages, salary from job",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="wages_value_known",
            field=models.CharField(
                choices=[
                    ("weekly", "as weekly income"),
                    ("monthly", "as monthly income"),
                    ("yearly", "as yearly income"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Over which <u>time period</u> are you able to estimate?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("9", "Self employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your employment status?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                verbose_name="What is your marital status?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_additional",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="Do you own any other property other than your primary dwelling?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_owner",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Don't know"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="Do you own any land.",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2023-01-23 00:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_consent",
            "0005_remove_subjectconsent_unique_consent_subject_id_and_version_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="group_identifier",
            field=models.CharField(max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="group_identifier",
            field=models.CharField(max_length=50, null=True),
        ),
    ]



import re
from typing import List

from django.apps import apps as django_apps
from django.db.models import Q
from django.urls import reverse
from edc_listboard.filters import ListboardFilter
from edc_listboard.filters import ListboardViewFilters as Base
from edc_listboard.views import ScreeningListboardView
from edc_sites import get_current_country

from intecomm_screening.constants import UGANDA

from ...model_wrappers import SubjectScreeningModelWrapper


class ListboardViewFilters(Base):
    all = ListboardFilter(name="all", label="All", lookup={})

    eligible = ListboardFilter(label="Eligible", position=10, lookup={"eligible": True})

    not_eligible = ListboardFilter(
        label="Not Eligible", position=11, lookup={"eligible": False}
    )

    consented = ListboardFilter(
        label="Consented", position=20, lookup={"eligible": True, "consented": True}
    )

    not_consented = ListboardFilter(
        label="Not consented",
        position=21,
        lookup={"eligible": True, "consented": False},
    )

    grouped = ListboardFilter(
        label="Grouped",
        position=30,
        lookup={
            "patient_log__patientgroup__isnull": False,
            "consented": True,
            "eligible": True,
        },
    )

    not_grouped = ListboardFilter(
        label="Not Grouped",
        position=31,
        lookup={
            "patient_log__patientgroup__isnull": True,
            "consented": True,
            "eligible": True,
        },
    )


class ListboardView(ScreeningListboardView):
    listboard_model = "intecomm_screening.subjectscreening"
    model_wrapper_cls = SubjectScreeningModelWrapper
    navbar_selected_item = "screen_group"
    listboard_view_filters = ListboardViewFilters()
    listboard_panel_title = "Screening"

    @property
    def listboard_model_cls(self):
        if get_current_country(request=self.request) == UGANDA:
            return django_apps.get_model("intecomm_screening.subjectscreeningug")
        return django_apps.get_model(self.listboard_model)

    def get_patient_log_add_url(self):
        if get_current_country(request=self.request) == UGANDA:
            return reverse("intecomm_screening_admin:intecomm_screening_patientlogug_add")
        return reverse("intecomm_screening_admin:intecomm_screening_patientlog_add")

    def get_context_data(self, **kwargs) -> dict:
        context_data = super().get_context_data(**kwargs)
        context_data.update(
            patient_log_add_url=self.get_patient_log_add_url(),
        )
        return context_data

    def extra_search_options(self, search_term) -> List[Q]:
        q_objects = super().extra_search_options(search_term)
        if re.match(r"^[0-9\-]+$", search_term):
            q_objects.append(Q(hospital_identifier__exact=search_term))
        return q_objects



from intecomm_rando.constants import FACILITY_ARM

from .base_subject_listboard_view import BaseSubjectListboardView


class FacilitySubjectListboardView(BaseSubjectListboardView):
    listboard_template = "facility_subject_listboard_template"
    listboard_url = "facility_subject_listboard_url"
    navbar_selected_item = "inte_followup"
    search_form_url = "facility_subject_listboard_url"
    assignment = FACILITY_ARM
    followup_url = "followup_facility_url"
    listboard_panel_title = "Facility-based participants"



from __future__ import annotations

import re
from typing import TYPE_CHECKING, Type

from django import forms
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from edc_constants.constants import UUID_PATTERN

if TYPE_CHECKING:
    from intecomm_consent.models import SubjectConsent, SubjectConsentUg
    from intecomm_screening.models import (
        PatientLog,
        SubjectScreening,
        SubjectScreeningUg,
    )


class AlreadyRefusedConsentError(Exception):
    pass


class InvalidScreeningIdentifier(Exception):
    pass


class InvalidSubjectIdentifier(Exception):
    pass


class MultipleConsentRefusalsDetectedError(Exception):
    pass


class ScreenedDespiteUnwillingToScreenError(Exception):
    pass


def get_consent_refusal_model_cls():
    return django_apps.get_model("intecomm_screening.consentrefusal")


def get_subject_screening_url(
    patient_log: PatientLog,
    subject_screening: SubjectScreening | None = None,
    subject_screening_model_cls: Type[SubjectScreening | SubjectScreeningUg] = None,
    next_url_name: str | None = None,
) -> str | None:
    if subject_screening:
        url = subject_screening.get_absolute_url()
    else:
        absolute_url = subject_screening_model_cls().get_absolute_url()
        url = f"{absolute_url}?patient_log_identifier={patient_log.patient_log_identifier}"
    if next_url_name:
        url = f"{url.split('?')[0]}?next={next_url_name}&{url.split('?')[1]}"
    return url


def get_subject_consent_url(
    subject_screening: SubjectScreening,
    subject_consent: SubjectConsent | SubjectConsentUg | None = None,
    subject_consent_model_cls: Type[SubjectConsent | SubjectConsentUg] = None,
    next_url_name: str | None = None,
) -> str:
    """Returns an add url if subject is screened eligible or a change
    url if subject consent exists.
    """
    url = None
    try:
        subject_consent = subject_consent or subject_consent_model_cls.objects.get(
            screening_identifier=subject_screening.screening_identifier
        )
    except ObjectDoesNotExist:
        if subject_screening and subject_screening.eligible:
            absolute_url = subject_consent_model_cls().get_absolute_url()
            url = (
                f"{absolute_url}?screening_identifier={subject_screening.screening_identifier}"
            )
    else:
        url = subject_consent.get_absolute_url()
    if next_url_name:
        url = f"{url.split('?')[0]}?next={next_url_name}&{url.split('?')[1]}"
    return url


def get_consent_refusal_url(
    screening_identifier: str,
    consent_refusal=None,
    next_url_name: str | None = None,
) -> str:
    try:
        consent_refusal = consent_refusal or get_consent_refusal_model_cls().objects.get(
            screening_identifier=screening_identifier
        )
    except ObjectDoesNotExist:
        url = get_consent_refusal_model_cls()().get_absolute_url()
        url = f"{url}?screening_identifier={screening_identifier}"
    else:
        url = consent_refusal.get_absolute_url()
    if next_url_name:
        url = f"{url.split('?')[0]}?next={next_url_name}&{url.split('?')[1]}"
    return url


def raise_if_consent_refusal_exists(
    screening_identifier: str, is_modelform: bool | None = None
) -> None:
    """Raises an exception if the consent refusal model instance
    exists.
    """
    try:
        get_consent_refusal_model_cls().objects.get(screening_identifier=screening_identifier)
    except ObjectDoesNotExist:
        pass
    else:
        if is_modelform:
            consent_refusal_url = get_consent_refusal_url(
                screening_identifier=screening_identifier
            )
            msg = format_html(
                "Not allowed. Patient has already refused consent. "
                'See subject <A href="{}">{}</A>',
                mark_safe(consent_refusal_url),  # nosec B308 B703
                screening_identifier,
            )
            raise forms.ValidationError(msg)

        else:
            raise AlreadyRefusedConsentError(
                f"Patient has already refused to consent. See {screening_identifier}. "
                "Perhaps catch this in the form."
            )


def validate_screening_identifier(
    screening_identifier: str,
    subject_screening_model_cls: Type[SubjectScreening | SubjectScreeningUg] = None,
    calling_model=None,
):
    """Raise if non-uuid identifier is not found in SubjectScreening."""
    if not screening_identifier or not re.match(UUID_PATTERN, screening_identifier):
        try:
            with transaction.atomic():
                subject_screening_model_cls.objects.get(
                    screening_identifier=screening_identifier
                )
        except ObjectDoesNotExist:
            raise InvalidScreeningIdentifier(
                f"Invalid screening identifier. See {calling_model._meta.verbose_name}. "
                f"Got `{screening_identifier}`. Perhaps catch this in the form."
            )


def validate_subject_identifier(
    subject_identifier,
    subject_consent_model_cls: Type[SubjectConsent | SubjectConsentUg] = None,
    calling_model=None,
):
    """Raise if non-uuid identifier is not found in SubjectConsent."""
    if not subject_identifier or not re.match(UUID_PATTERN, subject_identifier):
        try:
            with transaction.atomic():
                subject_consent_model_cls.objects.get(subject_identifier=subject_identifier)
        except ObjectDoesNotExist:
            raise InvalidSubjectIdentifier(
                f"Invalid subject identifier. See {calling_model._meta.verbose_name}. "
                f"Got `{subject_identifier}`. Perhaps catch this in the form."
            )



from edc_constants.constants import CLINIC, COMMUNITY
from edc_he.rule_groups import Predicates as BaseHealthEconomicsPredicates
from edc_visit_schedule.utils import is_baseline
from intecomm_rando.constants import COMMUNITY_ARM, FACILITY_ARM
from intecomm_rando.utils import get_assignment_for_subject


class HealthEconomicsPredicates(BaseHealthEconomicsPredicates):
    pass


class LocationUpdatePredicates:
    @staticmethod
    def location_needs_update(visit, **kwargs) -> bool:
        required = False
        try:
            appt_type_name = visit.appointment.appt_type.name
        except AttributeError:
            pass
        else:
            if not is_baseline(visit):
                assignment = (
                    COMMUNITY
                    if get_assignment_for_subject(visit.subject_identifier) == COMMUNITY_ARM
                    else CLINIC
                )
                if assignment != appt_type_name:
                    required = True
        return required


class NextAppointmentPredicates:
    @staticmethod
    def is_required(visit, **kwargs) -> bool:
        return get_assignment_for_subject(visit.subject_identifier) == FACILITY_ARM



from django.core.management import color_style
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from .visit_schedule import visit_schedule

style = color_style()

site_visit_schedules.register(visit_schedule)



# Generated by Django 4.2.5 on 2023-09-12 14:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0114_alter_dmmedicationadherence_visual_score_confirmed_and_more",
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalnextappointment",
            name="best_visit_code",
        ),
        migrations.RemoveField(
            model_name="historicalnextappointment",
            name="info_source_old",
        ),
        migrations.RemoveField(
            model_name="historicalnextappointment",
            name="old_health_facility",
        ),
        migrations.RemoveField(
            model_name="nextappointment",
            name="best_visit_code",
        ),
        migrations.RemoveField(
            model_name="nextappointment",
            name="info_source_old",
        ),
        migrations.RemoveField(
            model_name="nextappointment",
            name="old_health_facility",
        ),
        migrations.AddField(
            model_name="historicalsubjectvisitmissed",
            name="appt_date",
            field=models.DateField(
                help_text="Should fall on an Integrated clinic day",
                null=True,
                verbose_name="Next scheduled routine/facility appointment",
            ),
        ),
        migrations.AddField(
            model_name="subjectvisitmissed",
            name="appt_date",
            field=models.DateField(
                help_text="Should fall on an Integrated clinic day",
                null=True,
                verbose_name="Next scheduled routine/facility appointment",
            ),
        ),
    ]



from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class Medications(CrfModelMixin, BaseUuidModel):
    refill_hiv = models.CharField(
        verbose_name="Is the patient filling / refilling HIV medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Select `not applicable` if subject has not "
            "been diagnosed and prescribed medication for HIV infection."
        ),
    )

    refill_dm = models.CharField(
        verbose_name="Is the patient filling / refilling Diabetes medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Select `not applicable` if subject has not "
            "been diagnosed and prescribed medication for Diabetes."
        ),
    )

    refill_htn = models.CharField(
        verbose_name="Is the patient filling / refilling Hypertension medications?",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Select `not applicable` if subject has not "
            "been diagnosed and prescribed medication for Hypertension."
        ),
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Medications"
        verbose_name_plural = "Medications"



from .blood_results_fbc import BloodResultsFbc
from .blood_results_glu import BloodResultsGlu
from .blood_results_hba1c import BloodResultsHba1c
from .blood_results_ins import BloodResultsIns
from .blood_results_lft import BloodResultsLft
from .blood_results_lipid import BloodResultsLipid
from .blood_results_rft import BloodResultsRft



# Generated by Django 4.2.4 on 2023-08-28 17:10

import _socket
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_crypto_fields.fields.encrypted_char_field
import django_revision.revision_field
import edc_model.models.fields.duration
import edc_screening.model_mixins.screening_model_mixin
import edc_sites.model_mixins
import edc_utils.date
import edc_vitals.models.fields.blood_pressure
import simple_history.models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("intecomm_screening", "0058_patientlogug"),
    ]

    operations = [
        migrations.CreateModel(
            name="SubjectScreeningUg",
            fields=[],
            options={
                "verbose_name": "Subject Screening (Uganda)",
                "verbose_name_plural": "Subject Screening (Uganda)",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("intecomm_screening.subjectscreening",),
            managers=[
                (
                    "objects",
                    edc_screening.model_mixins.screening_model_mixin.ScreeningManager(),
                ),
                ("on_site", edc_sites.model_mixins.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalSubjectScreeningUg",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "legal_name",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Ensure full name consists of letters only in upper case separated by single spaces",
                                regex="^(([A-Z]+ )*[A-Z]+)?$",
                            )
                        ],
                        verbose_name="Full name",
                    ),
                ),
                (
                    "familiar_name",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Should be a name. Do NOT use MR, MRS, MISS, SIR, MADAM and other such titles. (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Ensure name consists of letters only in upper case separated by single spaces",
                                regex="^(([A-Z]+ )*[A-Z]+)?$",
                            )
                        ],
                        verbose_name="By what NAME should we refer to you? (if we speak to you)",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        db_index=True,
                        default="",
                        editable=False,
                        help_text="a field used for quick search",
                        max_length=250,
                        null=True,
                    ),
                ),
                ("eligible", models.BooleanField(default=False)),
                (
                    "reasons_ineligible",
                    models.TextField(
                        max_length=150, null=True, verbose_name="Reason not eligible"
                    ),
                ),
                (
                    "eligibility_datetime",
                    models.DateTimeField(
                        help_text="Date and time eligibility was determined relative to report_datetime",
                        null=True,
                    ),
                ),
                (
                    "real_eligibility_datetime",
                    models.DateTimeField(
                        help_text="Date and time eligibility was determined relative to now",
                        null=True,
                    ),
                ),
                (
                    "reference",
                    models.UUIDField(
                        db_index=True,
                        default=uuid.uuid4,
                        editable=False,
                        verbose_name="Reference",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="Date and time of report.",
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(110),
                        ]
                    ),
                ),
                (
                    "consent_ability",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Participant or legal guardian/representative able and willing to give informed consent.",
                    ),
                ),
                (
                    "unsuitable_for_study",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        help_text="If YES, patient NOT eligible, please give reason below.",
                        max_length=5,
                        verbose_name="Is there any other reason the patient is deemed to not be suitable for the study?",
                    ),
                ),
                (
                    "reasons_unsuitable",
                    models.TextField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not suitable for the study",
                    ),
                ),
                (
                    "unsuitable_agreed",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=5,
                        verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
                    ),
                ),
                ("consented", models.BooleanField(default=False, editable=False)),
                ("refused", models.BooleanField(default=False, editable=False)),
                (
                    "sys_blood_pressure_one",
                    edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                        blank=True, null=True
                    ),
                ),
                (
                    "dia_blood_pressure_one",
                    edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                        blank=True, null=True
                    ),
                ),
                (
                    "sys_blood_pressure_two",
                    edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                        blank=True, null=True
                    ),
                ),
                (
                    "dia_blood_pressure_two",
                    edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                        blank=True, null=True
                    ),
                ),
                (
                    "sys_blood_pressure_avg",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        verbose_name="Blood pressure: systolic (average)",
                    ),
                ),
                (
                    "dia_blood_pressure_avg",
                    models.IntegerField(
                        blank=True,
                        null=True,
                        verbose_name="Blood pressure: diastolic (average)",
                    ),
                ),
                (
                    "severe_htn",
                    models.CharField(
                        blank=True,
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="Based on the above readings. Severe HTN is any BP reading > 180/110mmHg",
                        max_length=15,
                        null=True,
                        verbose_name="Does the patient have severe hypertension?",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        blank=True, db_index=True, max_length=50, verbose_name="Screening ID"
                    ),
                ),
                (
                    "patient_log_identifier",
                    models.CharField(
                        db_index=True, max_length=50, verbose_name="Patient log identifier"
                    ),
                ),
                (
                    "selection_method",
                    models.CharField(
                        choices=[
                            ("random_sampling", "Random sampling"),
                            ("purposively_selected", "Purposively selected"),
                        ],
                        default="purposively_selected",
                        max_length=25,
                        verbose_name="How was the patient selected for screening?",
                    ),
                ),
                (
                    "hospital_identifier",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        db_index=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                    ),
                ),
                (
                    "lives_nearby",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Does the patient live within the catchment population of the health facility",
                    ),
                ),
                (
                    "staying_nearby_6",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Is the patient planning to remain in this catchment area for at least 6 months",
                    ),
                ),
                (
                    "in_care_6m",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="for 6m or more at the health facility from which they are being recruited",
                        max_length=15,
                        verbose_name="Has the patient been in regular care at this health facility for 6 months or more",
                    ),
                ),
                (
                    "in_care_duration",
                    edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
                ),
                (
                    "hiv_dx",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the patient previously been diagnosed with HIV",
                    ),
                ),
                (
                    "hiv_dx_6m",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=15,
                        verbose_name="Was the initial diagnosis made more than 6 months ago",
                    ),
                ),
                (
                    "hiv_dx_ago",
                    edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
                ),
                (
                    "art_unchanged_3m",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        help_text="Meaning no modifications to the type of medication and dose for at least 3 months",
                        max_length=15,
                        verbose_name="Has the patient been on the same anti-retroviral therapy for at least 3 months",
                    ),
                ),
                (
                    "art_stable",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        help_text="considered by the clinical team not to have any complications/co-infections or that these are well managed.",
                        max_length=15,
                        verbose_name="Is the patient considered to be stable on treatment ",
                    ),
                ),
                (
                    "art_adherent",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        help_text="in regular attendance for care",
                        max_length=15,
                        verbose_name="Is the patient considered to be adherent to treatment over the last 6 months",
                    ),
                ),
                (
                    "dm_dx",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the patient previously been diagnosed with Diabetes",
                    ),
                ),
                (
                    "dm_dx_6m",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=15,
                        verbose_name="Was the initial diagnosis made more than 6 months ago",
                    ),
                ),
                (
                    "dm_dx_ago",
                    edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
                ),
                (
                    "dm_complications",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=15,
                        verbose_name="Does the patient suffer from any complications of diabetes that are unmanaged or uncontrolled.",
                    ),
                ),
                (
                    "htn_dx",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the patient previously been diagnosed with Hypertension",
                    ),
                ),
                (
                    "htn_dx_6m",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=15,
                        verbose_name="Was the initial diagnosis made more than 6 months ago",
                    ),
                ),
                (
                    "htn_dx_ago",
                    edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
                ),
                (
                    "htn_complications",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        max_length=15,
                        verbose_name="Does the patient suffer from any complications of hypertension that are unmanaged or uncontrolled.",
                    ),
                ),
                (
                    "excluded_by_bp_history",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the patient's blood pressure exceeded 180/110 mmHg on more than one occasion any time in the last 6 months.",
                    ),
                ),
                (
                    "excluded_by_gluc_history",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the patient's blood glucose exceeded 13 mmol/L any time in the last 6 months.",
                    ),
                ),
                (
                    "requires_acute_care",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Does the patient have any clinical condition that requires health facility management",
                    ),
                ),
                (
                    "pregnant",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not Applicable: e.g. male or post-menopausal"),
                        ],
                        max_length=15,
                        verbose_name="Is the patient pregnant?",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "patient_log",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="intecomm_screening.patientlog",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="intecomm_screening.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Subject Screening (Uganda)",
                "verbose_name_plural": "historical Subject Screening (Uganda)",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from edc_search.model_mixins import SearchSlugModelMixin as Base


class SearchSlugModelMixin(Base):
    def get_search_slug_fields(self):
        fields = super().get_search_slug_fields()
        fields.append("subject_identifier")
        return fields

    class Meta:
        abstract = True



"""These models are not used by this trial. They are added to
maintain compatability with edc_adverse_event.
"""

from edc_adverse_event.model_mixins import (
    AeSusarModelMixin,
    AeTmgModelMixin,
    DeathReportTmgModelMixin,
)
from edc_model.models import BaseUuidModel


class AeSusar(AeSusarModelMixin, BaseUuidModel):
    class Meta(AeSusarModelMixin.Meta, BaseUuidModel.Meta):
        pass


class AeTmg(AeTmgModelMixin, BaseUuidModel):
    class Meta(AeTmgModelMixin.Meta, BaseUuidModel.Meta):
        pass


class DeathReportTmg(DeathReportTmgModelMixin, BaseUuidModel):
    class Meta(DeathReportTmgModelMixin.Meta, BaseUuidModel.Meta):
        pass


class DeathReportTmgSecond(DeathReportTmg):
    class Meta:
        proxy = True



# Generated by Django 4.2.3 on 2023-07-28 01:01

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0055_delete_identifierformat"),
    ]

    operations = [
        migrations.DeleteModel(
            name="HealthFacility",
        ),
    ]



from .ae_initial_model_wrapper import AeInitialModelWrapper
from .appointment_model_wrapper import AppointmentModelWrapper
from .consent_refusal_model_wrapper import ConsentRefusalModelWrapper
from .death_report_model_wrapper import DeathReportModelWrapper
from .patient_log_model_wrapper import PatientLogModelWrapper
from .subject_consent_model_wrappers import (
    SubjectConsentModelWrapper,
    SubjectConsentUgModelWrapper,
)
from .subject_screening_model_wrapper import (
    SubjectScreeningModelWrapper,
    SubjectScreeningUgModelWrapper,
)



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset

from ..admin_site import intecomm_subject_admin
from ..forms import ConcomitantMedicationForm
from ..models import ConcomitantMedication
from .modeladmin_mixins import CrfModelAdmin


@admin.register(ConcomitantMedication, site=intecomm_subject_admin)
class ConcomitantMedicationAdmin(CrfModelAdmin):
    form = ConcomitantMedicationForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                )
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )



from django import forms
from edc_form_validators import FormValidatorMixin
from intecomm_form_validators import PatientCallFormValidator

from ..models import PatientFollowupCall


class PatientFollowupCallForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = PatientCallFormValidator

    class Meta:
        model = PatientFollowupCall
        fields = "__all__"



from edc_constants.constants import COMMUNITY, DM, HIV, HTN, INTEGRATED, NCD, OTHER

list_data = {
    "edc_facility.healthfacilitytypes": [
        (COMMUNITY, "Community meeting place"),
        (INTEGRATED, "Facility Integrated Clinic"),
        (DM, "Diabetes Clinic"),
        (HIV, "HIV Clinic"),
        (HTN, "Hypertension Clinic"),
        (NCD, "NCD Clinic"),
        (OTHER, "Other (specify below)"),
    ],
}



# Generated by Django 4.2.7 on 2023-11-14 22:33
from django.db import migrations
from edc_appointment.constants import MISSED_APPT, NEW_APPT, ONTIME_APPT, SKIPPED_APPT
from edc_constants.constants import CLINIC, COMMUNITY, HOSPITAL, NOT_APPLICABLE
from tqdm import tqdm

from intecomm_visit_schedule.constants import COMMUNITY_SCHEDULE, FACILITY_SCHEDULE


def update_appointment(apps, schema_editor):
    appointment_model_cls = apps.get_model("edc_appointment.appointment")
    appointment_type_model_cls = apps.get_model("edc_appointment.appointmenttype")

    # update_appt_type if null in community
    appt_type_obj = appointment_type_model_cls.objects.get(name=COMMUNITY)
    qs = appointment_model_cls.objects.filter(
        appt_type_id__isnull=True, schedule_name=COMMUNITY_SCHEDULE
    )
    total = qs.count()
    for obj in tqdm(qs, total=total):
        obj.appt_type_id = appt_type_obj.id
        obj.save_base(update_fields=["appt_type_id"])

    # update_appt_type if null in facility
    appt_type_obj = appointment_type_model_cls.objects.get(name=CLINIC)
    qs = appointment_model_cls.objects.filter(
        appt_type_id__isnull=True, schedule_name=FACILITY_SCHEDULE
    )
    total = qs.count()
    for obj in tqdm(qs, total=total):
        obj.appt_type_id = appt_type_obj.id
        obj.save_base(update_fields=["appt_type_id"])

    # update_appt_status from skipped to new in_community
    qs = appointment_model_cls.objects.filter(
        appt_status=SKIPPED_APPT, schedule_name=COMMUNITY_SCHEDULE
    )
    total = qs.count()
    for obj in tqdm(qs, total=total):
        obj.appt_status = NEW_APPT
        obj.appt_timing = ONTIME_APPT
        obj.save()

    # appt_type from N/A to commmunity in community_arm
    appt_type_obj = appointment_type_model_cls.objects.get(name=NOT_APPLICABLE)
    qs = appointment_model_cls.objects.filter(
        appt_type_id=appt_type_obj.id, appt_status=NEW_APPT, schedule_name=COMMUNITY_SCHEDULE
    )
    total = qs.count()
    appt_type_obj = appointment_type_model_cls.objects.get(name=COMMUNITY)
    for obj in tqdm(qs, total=total):
        obj.appt_type_id = appt_type_obj.id
        obj.save()

    # appt_type from N/A to clinic in facility_arm
    appt_type_obj = appointment_type_model_cls.objects.get(name=NOT_APPLICABLE)
    qs = appointment_model_cls.objects.filter(
        appt_type_id=appt_type_obj.id, appt_status=NEW_APPT, schedule_name=FACILITY_SCHEDULE
    )
    total = qs.count()
    appt_type_obj = appointment_type_model_cls.objects.get(name=CLINIC)
    for obj in tqdm(qs, total=total):
        obj.appt_type_id = appt_type_obj.id
        obj.save()

    # appt_timing to ontime if appt_status is NEW
    appointment_model_cls.objects.filter(appt_status=NEW_APPT).update(appt_timing=ONTIME_APPT)

    # appt_type to clinic if facility appt is missed
    appt_type_obj = appointment_type_model_cls.objects.get(name=CLINIC)
    appointment_model_cls.objects.filter(
        appt_timing=MISSED_APPT, schedule_name=FACILITY_SCHEDULE
    ).update(appt_type_id=appt_type_obj.id)

    # appt_type to community if community appt is missed
    appt_type_obj = appointment_type_model_cls.objects.get(name=COMMUNITY)
    appointment_model_cls.objects.filter(
        appt_timing=MISSED_APPT, schedule_name=COMMUNITY_SCHEDULE
    ).update(appt_type_id=appt_type_obj.id)

    # appt_type to clinic if hospital
    hospital = appointment_type_model_cls.objects.get(name=HOSPITAL)
    clinic = appointment_type_model_cls.objects.get(name=CLINIC)
    appointment_model_cls.objects.filter(appt_type_id=hospital.id).update(
        appt_type_id=clinic.id
    )


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0123_alter_subjectvisit_managers"),
    ]

    operations = [migrations.RunPython(update_appointment)]



import re
from typing import Tuple

from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.utils.html import format_html
from edc_constants.choices import GENDER
from edc_constants.constants import UUID_PATTERN
from edc_model_admin.mixins import ModelAdminHideDeleteButtonOnCondition
from edc_screening.utils import format_reasons_ineligible
from edc_sites.admin import SiteModelAdminMixin

from ...utils import get_consent_refusal_model_cls
from ..modeladmin_mixins import InitialDataModelAdminMixin


class SubjectScreeningModelAdminMixin(
    SiteModelAdminMixin,
    InitialDataModelAdminMixin,
    ModelAdminHideDeleteButtonOnCondition,
):
    list_per_page = 5
    autocomplete_fields = ["site"]
    show_object_tools = True
    show_cancel = True

    change_list_template: str = "intecomm_screening/admin/subjectscreening_change_list.html"

    additional_instructions = (
        "Patients must meet ALL of the inclusion criteria and NONE of the "
        "exclusion criteria in order to proceed to the final screening stage"
    )

    list_display = (
        "screening_identifier",
        "eligibility_status",
        "demographics",
        "reasons",
        "report_datetime",
        "user_created",
        "created",
        "site_code",
    )

    list_filter = (
        "report_datetime",
        "gender",
        "eligible",
        "consented",
        "refused",
    )

    readonly_fields = (
        "screening_identifier",
        "eligible",
        "eligibility_datetime",
        "real_eligibility_datetime",
        "reasons_ineligible",
        "subject_identifier",
    )

    radio_fields = {
        "art_adherent": admin.VERTICAL,
        "art_stable": admin.VERTICAL,
        "art_unchanged_3m": admin.VERTICAL,
        "consent_ability": admin.VERTICAL,
        "dm_complications": admin.VERTICAL,
        "dm_dx": admin.VERTICAL,
        "dm_dx_6m": admin.VERTICAL,
        "excluded_by_bp_history": admin.VERTICAL,
        "excluded_by_gluc_history": admin.VERTICAL,
        "gender": admin.VERTICAL,
        "hiv_dx": admin.VERTICAL,
        "hiv_dx_6m": admin.VERTICAL,
        "htn_complications": admin.VERTICAL,
        "htn_dx": admin.VERTICAL,
        "htn_dx_6m": admin.VERTICAL,
        "in_care_6m": admin.VERTICAL,
        "lives_nearby": admin.VERTICAL,
        "pregnant": admin.VERTICAL,
        "requires_acute_care": admin.VERTICAL,
        "selection_method": admin.VERTICAL,
        "staying_nearby_6": admin.VERTICAL,
        "unsuitable_agreed": admin.VERTICAL,
        "unsuitable_for_study": admin.VERTICAL,
    }

    def patient_log_model_cls(self):
        pass

    def post_url_on_delete_kwargs(self, request, obj):
        return {}

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        readonly_fields = super().get_readonly_fields(request, obj=obj)
        if obj:
            readonly_fields = readonly_fields + ("patient_log",)
        return readonly_fields

    @staticmethod
    @admin.display(description="Eligibile", ordering="eligible")
    def eligibility_status(obj=None):
        return format_html('<span style="color:green;">YES</span>' if obj.eligible else "NO")

    @staticmethod
    def demographics(obj=None):
        data = [
            f"{obj.get_gender_display()} {obj.age_in_years}yrs",
            f"Initials: {obj.initials.upper()}<BR>",
            f"Hospital ID: {obj.hospital_identifier}",
            f"Patient log: {obj.patient_log_identifier}",
        ]
        return format_html("<BR>".join(data))

    def reasons(self, obj=None):
        if not obj.reasons_ineligible:
            return self.dashboard(obj)
        return format_reasons_ineligible(obj.reasons_ineligible)

    def hide_delete_button_on_condition(self, request, object_id) -> bool:
        """Hide delete button on form if subject identifier not
        set or consent_refusal exists.
        """
        obj = self.model.objects.get(id=object_id)
        try:
            consent_refusal = get_consent_refusal_model_cls().objects.get(
                screening_identifier=obj.screening_identifier
            )
        except ObjectDoesNotExist:
            consent_refusal = None
        if not re.match(UUID_PATTERN, obj.subject_identifier) or consent_refusal:
            return True
        return super().hide_delete_button_on_condition(request, object_id)

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "gender":
            kwargs["choices"] = GENDER
        return super().formfield_for_choice_field(db_field, request, **kwargs)

    def get_changeform_initial_data(self, request) -> dict:
        dct = super().get_changeform_initial_data(request)
        try:
            patient_log = self.patient_log_model_cls().objects.get(
                patient_log_identifier=dct.get("patient_log_identifier")
            )
        except ObjectDoesNotExist:
            pass
        else:
            dct.update(**self.initial_form_data(request, patient_log))
        return dct



from __future__ import annotations

from django.apps import apps as django_apps
from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_model_admin.history import SimpleHistoryAdmin

from intecomm_sites import all_sites

from ..admin_site import intecomm_consent_admin
from ..models import SubjectConsentUg
from .fieldsets import (
    get_first_fieldset,
    get_group_fieldset,
    get_review_questions_fieldset,
)
from .modeladmin_mixins import SubjectConsentModelAdminMixin


@admin.register(SubjectConsentUg, site=intecomm_consent_admin)
class SubjectConsentUgAdmin(
    SubjectConsentModelAdminMixin,
    SimpleHistoryAdmin,
):
    all_sites = all_sites

    # ModelAdminRedirectAllToChangelistMixin
    changelist_url = "intecomm_screening_admin:intecomm_screening_patientlogug_changelist"
    change_search_field_name = "screening_identifier"
    add_search_field_name = "screening_identifier"

    fieldsets = (
        get_first_fieldset(include_pii=False),
        get_review_questions_fieldset(),
        get_group_fieldset(),
        audit_fieldset_tuple,
    )

    search_fields = (
        "subject_identifier",
        "screening_identifier",
        "identity__exact",
        "initials__exact",
    )

    @property
    def subject_screening_model_cls(self):
        return django_apps.get_model("intecomm_screening.subjectscreeningug")



from django.db import models
from edc_consent.utils import get_consent_model_cls
from edc_model.models import BaseUuidModel
from edc_model.models.historical_records import HistoricalRecords
from edc_model_fields.fields.other_charfield import OtherCharField
from edc_screening.utils import get_subject_screening_or_raise
from edc_search.model_mixins import SearchSlugManager
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow

from intecomm_consent.utils import raise_if_subject_consent_exists
from intecomm_lists.models import ConsentRefusalReasons

from .subject_screening import SubjectScreening


class ConsentRefusalManager(SearchSlugManager, models.Manager):
    def get_by_natural_key(self, screening_identifier):
        return self.get(screening_identifier=screening_identifier)


class ConsentRefusal(SiteModelMixin, BaseUuidModel):
    subject_screening = models.OneToOneField(
        SubjectScreening, on_delete=models.PROTECT, blank=True
    )

    screening_identifier = models.CharField(max_length=36, unique=True)

    report_datetime = models.DateTimeField(
        verbose_name="Report date and time", default=get_utcnow
    )

    reason = models.ForeignKey(
        ConsentRefusalReasons,
        on_delete=models.PROTECT,
        verbose_name="Reason for refusal to consent",
        max_length=25,
        null=True,
        blank=False,
    )

    other_reason = OtherCharField()

    on_site = CurrentSiteManager()
    objects = ConsentRefusalManager()
    history = HistoricalRecords()

    def save(self, *args, **kwargs):
        self.subject_screening = get_subject_screening_or_raise(self.screening_identifier)
        raise_if_subject_consent_exists(
            screening_identifier=self.screening_identifier,
            subject_consent_model_cls=get_consent_model_cls(),
        )
        super().save(*args, **kwargs)

    def __str__(self):
        return (
            f"{self.screening_identifier} {self.subject_screening.gender} "
            f"{self.subject_screening.age_in_years}"
        )

    def natural_key(self):
        return (self.screening_identifier,)

    def get_search_slug_fields(self):
        return ["screening_identifier"]

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Consent Refusal"
        verbose_name_plural = "Consent Refusals"



# Generated by Django 4.1.2 on 2023-01-27 01:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0009_remove_historicalvitals_bp_measured_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalvitals",
            name="weight_is_estmated",
        ),
        migrations.RemoveField(
            model_name="vitals",
            name="weight_is_estmated",
        ),
        migrations.AddField(
            model_name="historicalvitals",
            name="comments",
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name="historicalvitals",
            name="weight_determination",
            field=models.CharField(
                choices=[
                    ("measured", "Measured"),
                    ("estimated", "Estimated"),
                    ("N/A", "Not recorded / Not applicable"),
                ],
                default="measured",
                max_length=15,
                verbose_name="Is weight estimated or measured?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="vitals",
            name="comments",
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name="vitals",
            name="weight_determination",
            field=models.CharField(
                choices=[
                    ("measured", "Measured"),
                    ("estimated", "Estimated"),
                    ("N/A", "Not recorded / Not applicable"),
                ],
                default="meqsured",
                max_length=15,
                verbose_name="Is weight estimated or measured?",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="historicalvitals",
            name="bp_one_not_taken_reason",
            field=models.CharField(
                blank=True,
                max_length=50,
                null=True,
                verbose_name="If not done, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="historicalvitals",
            name="bp_two_not_taken_reason",
            field=models.CharField(
                blank=True,
                max_length=50,
                null=True,
                verbose_name="If not done, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="vitals",
            name="bp_one_not_taken_reason",
            field=models.CharField(
                blank=True,
                max_length=50,
                null=True,
                verbose_name="If not done, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="vitals",
            name="bp_two_not_taken_reason",
            field=models.CharField(
                blank=True,
                max_length=50,
                null=True,
                verbose_name="If not done, please explain",
            ),
        ),
    ]



import os
from getpass import getpass

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.exceptions import ObjectDoesNotExist
from django.core.management import BaseCommand, CommandError
from edc_sites import get_sites_by_country
from edc_utils import get_utcnow
from tqdm import tqdm

from intecomm_screening.models import PatientLog, PatientLogReportPrintHistory
from intecomm_screening.reports import PatientLogReport


class Command(BaseCommand):
    def __init__(self, **kwargs):
        self.sites: list[int] = []
        super().__init__(**kwargs)

    def add_arguments(self, parser):
        parser.add_argument(
            "-p",
            "--path",
            dest="path",
            default=False,
            help="export path",
        )

        parser.add_argument(
            "--country",
            dest="country",
            default="",
            help="only export data for country",
        )

        parser.add_argument(
            "--site",
            dest="site",
            default="",
            help="only export data for site",
        )

    def handle(self, *args, **options):
        user = self.validate_user_perms_or_raise()
        export_path = os.path.expanduser(options["path"])
        if not export_path or not os.path.exists(export_path):
            raise CommandError(f"Path does not exist. Got `{export_path}`")
        sites = [options["site"]] if options["site"] else []
        countries = [options["country"]] if options["country"] else []
        self.sites = self.get_sites(sites=sites, countries=countries)
        queryset = PatientLog.objects.filter(site__in=self.sites).order_by(
            "site", "filing_identifier"
        )
        total = queryset.count()
        for obj in tqdm(queryset, total=total):
            PatientLogReport(patient_log=obj, user=user).render_to_file(
                export_path, verbose=False
            )
            PatientLogReportPrintHistory.objects.create(
                patient_log_identifier=obj.patient_log_identifier,
                printed_datetime=get_utcnow(),
                printed_user=user.username,
            )
            obj.printed = True
            obj.save(update_fields=["printed"])

    @staticmethod
    def validate_user_perms_or_raise() -> User:
        username = input("Username:")
        passwd = getpass("Password for " + username + ":")
        try:
            user = User.objects.get(username=username, is_superuser=False, is_active=True)
        except ObjectDoesNotExist:
            raise CommandError("Invalid username or password.")
        if not user.check_password(passwd):
            raise CommandError("Invalid username or password.")
        if not user.groups.filter(name="EXPORT_PII").exists():
            raise CommandError("You are not authorized to export/print sensitive data.")
        return user

    @staticmethod
    def get_sites(
        sites: list[int] | list[str] | None,
        countries: list[str] | None,
    ) -> list[int]:
        if countries and sites:
            raise CommandError("Invalid. Specify `sites` or `countries`, not both.")
        for site in sites or []:
            try:
                obj = Site.objects.get(id=int(site))
            except ObjectDoesNotExist:
                raise CommandError(f"Invalid site. Got `{site}`.")
            else:
                sites.append(obj.id)
        for country in countries or []:
            for single_site in get_sites_by_country(country):
                sites.append(single_site.site_id)
        return sites



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_constants.constants import YES
from edc_form_validators.form_validator import FormValidator
from edc_form_validators.form_validator_mixins import FormValidatorMixin
from edc_model_form.mixins import BaseModelFormMixin
from edc_offstudy.modelform_mixins import OffstudyNonCrfModelFormMixin
from edc_prn.modelform_mixins import PrnFormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from ..models import LossToFollowup


class LossToFollowupFormValidator(PrnFormValidatorMixin, FormValidator):
    def clean(self):
        # TODO: should not be submitted before 12 months from baseline
        self.required_if(YES, field="home_visit", field_required="home_visit_detail")
        self.validate_other_specify(
            field="loss_category", other_specify_field="loss_category_other"
        )


class LossToFollowupForm(
    SiteModelFormMixin,
    OffstudyNonCrfModelFormMixin,
    ActionItemFormMixin,
    BaseModelFormMixin,
    FormValidatorMixin,
    forms.ModelForm,
):
    form_validator_cls = LossToFollowupFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = LossToFollowup
        fields = "__all__"



# Generated by Django 4.1.2 on 2022-11-22 22:52

import django.core.validators
from django.db import migrations, models
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_consent", "0001_initial"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="subjectconsent",
            name="intecomm_co_subject_145905_idx",
        ),
        migrations.AlterUniqueTogether(
            name="subjectconsent",
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name="historicalsubjectconsent",
            name="first_name",
        ),
        migrations.RemoveField(
            model_name="historicalsubjectconsent",
            name="last_name",
        ),
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsent",
            name="legal_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                db_index=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure full name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="Full name",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsent",
            name="legal_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure full name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="Full name",
            ),
        ),
        migrations.AddIndex(
            model_name="subjectconsent",
            index=models.Index(
                fields=["subject_identifier", "legal_name", "dob", "initials", "version"],
                name="intecomm_co_subject_c503bc_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="subjectconsent",
            constraint=models.UniqueConstraint(
                models.F("subject_identifier"),
                models.F("version"),
                name="unique_consent_subject_id_and_version",
                violation_error_message="A subject with this identifier has already completed this version of the consent",
            ),
        ),
        migrations.AddConstraint(
            model_name="subjectconsent",
            constraint=models.UniqueConstraint(
                models.F("subject_identifier"),
                models.F("screening_identifier"),
                name="unique_consent_subject_id_screening_id",
            ),
        ),
        migrations.AddConstraint(
            model_name="subjectconsent",
            constraint=models.UniqueConstraint(
                models.F("familiar_name"),
                models.F("dob"),
                models.F("initials"),
                models.F("version"),
                name="unique_consent_name_dob_initials",
                violation_error_message="A subject with this 'familiar' name, dob and initials has already completed this version of the consent",
            ),
        ),
        migrations.RemoveField(
            model_name="subjectconsent",
            name="first_name",
        ),
        migrations.RemoveField(
            model_name="subjectconsent",
            name="last_name",
        ),
    ]



# Generated by Django 4.2.4 on 2023-08-16 21:30

from django.db import migrations
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0105_alter_historicallocationupdate_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="dmmedicationadherence",
            name="meds_shortage_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If `other` reason for not having medications, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicaldmmedicationadherence",
            name="meds_shortage_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If `other` reason for not having medications, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivmedicationadherence",
            name="meds_shortage_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If `other` reason for not having medications, specify",
            ),
        ),
        migrations.AddField(
            model_name="historicalhtnmedicationadherence",
            name="meds_shortage_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If `other` reason for not having medications, specify",
            ),
        ),
        migrations.AddField(
            model_name="hivmedicationadherence",
            name="meds_shortage_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If `other` reason for not having medications, specify",
            ),
        ),
        migrations.AddField(
            model_name="htnmedicationadherence",
            name="meds_shortage_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If `other` reason for not having medications, specify",
            ),
        ),
    ]



from __future__ import annotations

from django.apps import apps as django_apps
from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_model_admin.mixins import ModelAdminRedirectAllToChangelistMixin

from intecomm_sites import all_sites

from ...admin_site import intecomm_screening_admin
from ...forms import PatientLogForm
from ...models import PatientLog
from ..modeladmin_mixins import BaseModelAdminMixin
from .fieldsets import (
    get_address_fielset,
    get_appointment_fieldset,
    get_contact_fieldset,
    get_first_fieldset,
    get_group_fieldset,
    get_health_fieldset,
    get_health_talks_fieldset,
    get_names_and_basic_demographics_fieldset,
    get_screening_and_consent_fieldset,
    get_willingness_to_screen_fieldset,
)
from .get_search_fields import get_search_fields
from .modeladmin_mixins import PatientLogModelAdminMixin


@admin.register(PatientLog, site=intecomm_screening_admin)
class PatientLogAdmin(
    PatientLogModelAdminMixin,
    ModelAdminRedirectAllToChangelistMixin,
    BaseModelAdminMixin,
):
    # ChangeListTopBarModelAdminMixin attrs
    changelist_top_bar_selected = "patientlog"
    changelist_top_bar_add_url = "intecomm_screening_admin:intecomm_screening_patientlog_add"
    change_list_template: str = "intecomm_screening/admin/patientlog_change_list.html"

    # ModelAdminRedirectAllToChangelistMixin
    changelist_url = "intecomm_screening_admin:intecomm_screening_patientlog_changelist"
    change_search_field_name = "screening_identifier"
    add_search_field_name = "screening_identifier"

    change_list_title = PatientLog._meta.verbose_name

    all_sites = all_sites

    form = PatientLogForm

    fieldsets = (
        get_first_fieldset(),
        get_names_and_basic_demographics_fieldset(include_pii=True),
        get_contact_fieldset(),
        get_address_fielset(),
        get_health_fieldset(),
        get_health_talks_fieldset(),
        get_appointment_fieldset(),
        get_willingness_to_screen_fieldset(),
        get_screening_and_consent_fieldset(),
        get_group_fieldset(),
        audit_fieldset_tuple,
    )

    search_fields = get_search_fields(include_pii=True)

    @property
    def subject_screening_model_cls(self):
        return django_apps.get_model("intecomm_screening.subjectscreening")

    @property
    def subject_consent_model_cls(self):
        return django_apps.get_model("intecomm_consent.subjectconsent")



# Generated by Django 4.1.7 on 2023-05-11 16:22

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0030_auto_20230510_0352"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="next_appt_date",
            field=models.DateField(
                blank=True,
                help_text="If known, this date will help prioritize efforts to contact the patient",
                null=True,
                verbose_name="Next scheduled routine appointment at this health facility",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="next_appt_date",
            field=models.DateField(
                blank=True,
                help_text="If known, this date will help prioritize efforts to contact the patient",
                null=True,
                verbose_name="Next scheduled routine appointment at this health facility",
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_adverse_event.modeladmin_mixins import (
    AeInitialModelAdminMixin,
    fieldset_part_three,
)
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_ae_admin
from ..forms import AeInitialForm
from ..models import AeInitial


@admin.register(AeInitial, site=intecomm_ae_admin)
class AeInitialAdmin(SiteModelAdminMixin, AeInitialModelAdminMixin, SimpleHistoryAdmin):
    form = AeInitialForm
    additional_instructions = None

    fieldsets = (
        (None, {"fields": ("subject_identifier", "report_datetime")}),
        (
            "Part 1: Description",
            {
                "fields": (
                    "ae_classification_as_text",
                    "ae_description",
                    "ae_awareness_date",
                    "ae_start_date",
                    "ae_grade",
                )
            },
        ),
        fieldset_part_three,
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "ae_grade": admin.VERTICAL,
    }

    def get_list_filter(self, request) -> tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("ae_awareness_date", "ae_grade")
        excluded_fields = [
            "ae_classification",
            "sae",
            "sae_reason",
            "susar",
            "susar_reported",
        ]
        excluded_fields.extend(custom_fields)
        list_filter = custom_fields + tuple(f for f in list_filter if f not in excluded_fields)
        # TODO: remove with Django > 4.2.5
        self.list_filter = list_filter
        return list_filter



from edc_crf.modelform_mixins import CrfModelFormMixin as BaseCrfModelFormMixin
from edc_dx_review.form_mixins import ClinicalReviewBaselineRequiredModelFormMixin


class CrfModelFormMixin(ClinicalReviewBaselineRequiredModelFormMixin, BaseCrfModelFormMixin):
    pass



from django import forms
from intecomm_form_validators.subject import DrugRefillHivFormValidator

from ..models import DrugRefillHiv
from .mixins import CrfModelFormMixin


class DrugRefillHivForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DrugRefillHivFormValidator

    class Meta:
        model = DrugRefillHiv
        fields = "__all__"



from edc_model_wrapper import ModelWrapper


class SubjectConsentModelWrapper(ModelWrapper):
    model: str = "intecomm_consent.subjectconsent"
    next_url_name = "subject_dashboard_url"
    next_url_attrs = ["subject_identifier"]
    querystring_attrs = [
        "screening_identifier",
        "gender",
        "modified",
    ]


class SubjectConsentUgModelWrapper(ModelWrapper):
    model: str = "intecomm_consent.subjectconsentug"
    next_url_name = "subject_dashboard_url"
    next_url_attrs = ["subject_identifier"]
    querystring_attrs = [
        "screening_identifier",
        "gender",
        "modified",
    ]



from django.db import models
from edc_facility.model_mixins import HealthFacilityModelMixin, Manager
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.models import CurrentSiteManager, SiteModelMixin


class HealthFacility(SiteModelMixin, HealthFacilityModelMixin, BaseUuidModel):
    distance = models.IntegerField(
        verbose_name="Approximate distance from integrated-care clinic",
        help_text="In km. If within the integrated-care clinic facility type '0'",
    )

    objects = Manager()
    on_site = CurrentSiteManager()
    history = HistoricalRecords()

    class Meta(SiteModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Health Facility"
        verbose_name_plural = "Health Facilities"



from .subject_screening import SubjectScreening


class SubjectScreeningUg(SubjectScreening):
    class Meta:
        proxy = True
        verbose_name = "Subject Screening (Uganda)"
        verbose_name_plural = "Subject Screening (Uganda)"



from django.contrib import admin
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch
from django_audit_fields import audit_fields, audit_fieldset_tuple
from edc_dashboard.url_names import url_names
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from intecomm_subject.models import SubjectVisit

from ..admin_site import intecomm_consent_admin
from ..forms import SubjectReconsentForm
from ..models import SubjectReconsent


@admin.register(SubjectReconsent, site=intecomm_consent_admin)
class SubjectReconsentAdmin(
    SiteModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):
    form = SubjectReconsentForm

    fieldsets = (
        (None, {"fields": ("subject_identifier", "identity")}),
        (
            "Review Questions",
            {
                "fields": (
                    "consent_reviewed",
                    "study_questions",
                    "assessment_score",
                    "consent_signature",
                    "consent_copy",
                ),
                "description": "The following questions are directed to the interviewer.",
            },
        ),
        audit_fieldset_tuple,
    )

    search_fields = ("subject_identifier", "identity")

    radio_fields = {
        "assessment_score": admin.VERTICAL,
        "consent_copy": admin.VERTICAL,
        "consent_reviewed": admin.VERTICAL,
        "consent_signature": admin.VERTICAL,
        "study_questions": admin.VERTICAL,
    }

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super().get_readonly_fields(request, obj=obj)
        return list(readonly_fields) + list(audit_fields)

    def view_on_site(self, obj):
        url_name = url_names.get("subject_dashboard_url")
        try:
            return reverse(url_name, kwargs=dict(subject_identifier=obj.subject_identifier))
        except NoReverseMatch:
            return super().view_on_site(obj)

    def delete_view(self, request, object_id, extra_context=None):
        """Prevent deletion if SubjectVisit objects exist."""
        extra_context = extra_context or {}
        obj = SubjectReconsent.objects.get(id=object_id)
        try:
            protected = [SubjectVisit.objects.get(subject_identifier=obj.subject_identifier)]
        except ObjectDoesNotExist:
            protected = None
        except MultipleObjectsReturned:
            protected = SubjectVisit.objects.filter(subject_identifier=obj.subject_identifier)
        extra_context.update({"protected": protected})
        return super().delete_view(request, object_id, extra_context)



from .appointment_admin import AppointmentAdmin
from .arv_regimens_admin import ArvRegimensAdmin
from .blood_results import (
    BloodResultsFbcAdmin,
    BloodResultsGluAdmin,
    BloodResultsHba1cAdmin,
    BloodResultsInsAdmin,
    BloodResultsLftAdmin,
    BloodResultsLipidAdmin,
    BloodResultsRftAdmin,
)
from .cd4_result_admin import Cd4ResultAdmin
from .clinic_note_admin import ClinicalNoteAdmin
from .clinical_review_admin import ClinicalReviewAdmin
from .clinical_review_baseline_admin import ClinicalReviewBaselineAdmin
from .complications_baseline_admin import ComplicationsBaselineAdmin
from .complications_followup_admin import ComplicationsFollowupAdmin
from .concomitant_medication_admin import ConcomitantMedicationAdmin
from .dm_initial_review_admin import DmInitialReviewAdmin
from .dm_medication_adherence_admin import DmMedicationAdherenceAdmin
from .dm_review_admin import DmReviewAdmin
from .drug_refill_dm_admin import DrugRefillDmAdmin
from .drug_refill_hiv_admin import DrugRefillHivAdmin
from .drug_refill_htn_admin import DrugRefillHtnAdmin
from .eq5d3l_admin import Eq5d3lAdmin
from .family_history_admin import FamilyHistoryAdmin
from .glucose_admin import GlucoseAdmin
from .health_economics import (
    HealthEconomicsAssetsAdmin,
    HealthEconomicsHouseholdHeadAdmin,
    HealthEconomicsIncomeAdmin,
    HealthEconomicsPatientAdmin,
    HealthEconomicsPropertyAdmin,
)
from .hiv_initial_review_admin import HivInitialReviewAdmin
from .hiv_medication_adherence_admin import HivMedicationAdherenceAdmin
from .hiv_review_admin import HivReviewAdmin
from .htn_initial_review_admin import HtnInitialReviewAdmin
from .htn_medication_adherence_admin import HtnMedicationAdherenceAdmin
from .htn_review_admin import HtnReviewAdmin
from .icecapa_admin import IcecapaModelAdmin
from .location_update_admin import LocationUpdateAdmin
from .malaria_test_admin import MalariaTestAdmin
from .medications_admin import MedicationsAdmin
from .next_appointment_admin import NextAppointmentAdmin
from .other_baseline_data_admin import OtherBaselineDataAdmin
from .social_harms_admin import SocialHarmsAdmin
from .subject_requisition_admin import SubjectRequisitionAdmin
from .subject_visit_admin import SubjectVisitAdmin
from .subject_visit_missed_admin import SubjectVisitMissedAdmin
from .urine_dipstick_test_admin import UrineDipstickTestAdmin
from .urine_pregnancy_admin import UrinePregnancyAdmin
from .viral_load_result_admin import ViralLoadResultAdmin
from .vitals_admin import VitalsAdmin



from edc_adverse_event.model_mixins import AeFollowupModelMixin
from edc_model.models.base_uuid_model import BaseUuidModel


class AeFollowup(AeFollowupModelMixin, BaseUuidModel):
    class Meta(AeFollowupModelMixin.Meta, BaseUuidModel.Meta):
        pass



from decimal import Decimal
from unittest.mock import patch
from uuid import uuid4

from django.db import IntegrityError
from django.test import TestCase
from edc_constants.constants import (
    DM,
    FEMALE,
    HIV,
    HTN,
    MALE,
    NO,
    TBD,
    UUID_PATTERN,
    YES,
)
from intecomm_form_validators import RECRUITING
from model_bakery.baker import make_recipe

from intecomm_lists.models import Conditions, ScreeningRefusalReasons
from intecomm_screening.forms import PatientLogForm, SubjectScreeningForm
from intecomm_screening.models import PatientLog, SubjectScreening
from intecomm_screening.models.subject_screening import SubjectScreeningError
from intecomm_screening.utils import InvalidScreeningIdentifier

from ...constants import UGANDA
from ..intecomm_test_case_mixin import IntecommTestCaseMixin


class TestScreening(IntecommTestCaseMixin, TestCase):
    def test_no_patients_ok(self):
        obj = make_recipe("intecomm_screening.patientgroup")
        self.assertEqual(obj.status, RECRUITING)
        self.assertIsNotNone(obj.name)
        self.assertIsNotNone(obj.group_identifier)
        self.assertRegexpMatches(str(obj.group_identifier), UUID_PATTERN)
        self.assertEqual(obj.ratio, Decimal("0"))
        self.assertEqual(obj.randomize_now, NO)
        self.assertFalse(obj.randomized)
        self.assertEqual(obj.patients.all().count(), 0)

    def test_with_patients_ok(self):
        obj = None
        initials = "ABCDEFGHIJKLMNOP"
        for i in range(0, 4):
            obj = make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEA{i} AAA{i}",
                familiar_name=f"NAMEA{i}",
                initials=f"N{initials[i]}A",
                hospital_identifier=uuid4().hex,
                contact_number=f"123456789{i}",
            )
            obj.conditions.add(Conditions.objects.get(name=HIV))
        for i in range(0, 5):
            make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEB{i} BBB{i}",
                familiar_name=f"NAMEB{i}",
                hospital_identifier=uuid4().hex,
                contact_number=f"12345678{i}9",
            )
            obj.conditions.add(Conditions.objects.get(name=DM))
        for i in range(0, 5):
            make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEC{i} CCC{i}",
                familiar_name=f"NAMEC{i}",
                hospital_identifier=uuid4().hex,
                contact_number=f"1234567{i}89",
            )
            obj.conditions.add(Conditions.objects.get(name=HTN))
        for patient_log in PatientLog.objects.all():
            self.assertEqual(patient_log.patientgroup_set.all().count(), 0)
        self.assertEqual(PatientLog.objects.all().count(), 14)

        # create patient_group
        patient_group = make_recipe("intecomm_screening.patientgroup")

        # add patient_logs to patient_group
        for index, patient_log in enumerate(PatientLog.objects.all()):
            patient_group.patients.add(patient_log)
        for patient_log in PatientLog.objects.all():
            self.assertEqual(patient_log.patientgroup_set.all().count(), 1)
            self.assertEqual(patient_log.patientgroup_set.all().first(), patient_group)

        patient_group.refresh_from_db()

        # TODO: why does this fail?
        # self.assertEqual(patient_group.patients.count(), 14)

    def test_screening_raises_if_unwilling_in_patient_log(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier=uuid4().hex,
            contact_number="1234567890",
            willing_to_screen=NO,
            screening_refusal_reason=ScreeningRefusalReasons.objects.get(
                name="dont_have_time"
            ),
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier=patient_log.hospital_identifier,
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials=patient_log.initials,
        )

        with self.assertRaises(SubjectScreeningError) as cm:
            subject_screening.save()
        self.assertIn(
            f"Patient '{patient_log.patient_log_identifier}' is unwilling to screen.",
            str(cm.exception),
        )

    def test_screening_ok_if_not_unwilling_in_patient_log(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier=uuid4().hex,
            contact_number="1234567890",
            willing_to_screen=YES,
            screening_refusal_reason=None,
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier=patient_log.hospital_identifier,
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials=patient_log.initials,
        )

        try:
            subject_screening.save()
        except SubjectScreeningError as e:
            self.fail(f"SubjectScreeningError unexpectedly raised. Got {e}")

    def test_screening_raises_if_hospital_identifier_mismatch(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier="56d2c5ebc7384309990ddc14ee2cf1b2",
            contact_number="1234567890",
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier="e8da3474883445cd906697d91e52a10a",
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials=patient_log.initials,
        )

        with self.assertRaises(SubjectScreeningError) as cm:
            subject_screening.save()
        self.assertIn(
            "Health facility identifier does not match patient log. ",
            str(cm.exception),
        )

    def test_screening_raises_if_initials_mismatch(self):
        patient_log = make_recipe(
            "intecomm_screening.patientlog",
            legal_name="NAMEA AAA",
            familiar_name="NAMEA",
            initials="NA",
            hospital_identifier=uuid4().hex,
            contact_number="1234567890",
        )
        patient_log.conditions.add(Conditions.objects.get(name=HIV))
        subject_screening = SubjectScreening(
            patient_log_identifier=patient_log.patient_log_identifier,
            hospital_identifier=patient_log.hospital_identifier,
            gender=patient_log.gender,
            age_in_years=patient_log.age_in_years,
            legal_name=patient_log.legal_name,
            familiar_name=patient_log.familiar_name,
            initials="XX",
        )

        with self.assertRaises(SubjectScreeningError) as cm:
            subject_screening.save()
        self.assertIn(
            "Initials do not match patient log. ",
            str(cm.exception),
        )

    def test_patient_log_gender_cannot_be_changed_after_screen(self):
        patient_log = self.get_patient_log(willing_to_screen=YES, gender=FEMALE)
        self.get_subject_screening(patient_log=patient_log, gender=FEMALE)
        form = PatientLogForm(data=dict(gender=MALE), instance=patient_log)
        form.is_valid()
        self.assertIn("Patient has already screened. Gender", str(form._errors.get("__all__")))

    def test_patient_log_initials_cannot_be_changed_after_screen(self):
        patient_log = self.get_patient_log(willing_to_screen=YES, gender=FEMALE, initials="YY")
        self.get_subject_screening(patient_log=patient_log, gender=FEMALE)
        form = PatientLogForm(data=dict(gender=FEMALE, initials="XX"), instance=patient_log)
        form.is_valid()
        self.assertIn(
            "Patient has already screened. Initials", str(form._errors.get("__all__"))
        )

    def test_screening_id_not_found_in_patient_log_raises(self):
        patient_log = self.get_patient_log()
        self.get_subject_screening(patient_log=patient_log)
        patient_log.screening_identifier = "some_other_identifier"
        self.assertRaises(InvalidScreeningIdentifier, patient_log.save)

    def test_patient_log_screening_identifier_unique_constraint(self):
        patient_log = self.get_patient_log()
        self.get_subject_screening(patient_log=patient_log)
        patient_log_two = self.get_patient_log(legal_name="NOTHER AME")
        patient_log_two.screening_identifier = patient_log.screening_identifier
        self.assertRaises(IntegrityError, patient_log_two.save)

    @patch("intecomm_screening.forms.subject_screening.modelform_mixins.get_current_country")
    def test_unwilling_to_screen_in_patient_log_raises(self, mock_get_current_country):
        mock_get_current_country.result = "uganda"
        patient_log = self.get_patient_log(
            willing_to_screen=NO,
            screening_refusal_reason=ScreeningRefusalReasons.objects.get(
                name="dont_have_time"
            ),
            gender=FEMALE,
        )

        cleaned_data = dict(
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=FEMALE,
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Patient is unwilling to screen", str(form._errors.get("__all__", "")))

        patient_log.willing_to_screen = YES
        patient_log.screening_refusal_reason = None
        patient_log.save()
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn(
            "Patient is unwilling to screen", str(form._errors.get("__all__", ""))
        )

    @patch("intecomm_screening.forms.subject_screening.modelform_mixins.get_current_country")
    def test_gender(self, mock_get_current_country):
        mock_get_current_country.result = "uganda"
        patient_log = self.get_patient_log(
            gender=FEMALE,
            willing_to_screen=YES,
            screening_refusal_reason=None,
        )
        cleaned_data = dict(
            gender=MALE, patient_log_identifier=patient_log.patient_log_identifier
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Invalid. Expected Female", str(form._errors.get("gender", "")))

        cleaned_data = dict(
            gender=FEMALE, patient_log_identifier=patient_log.patient_log_identifier
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn("gender", form._errors)

    @patch("intecomm_screening.forms.subject_screening.modelform_mixins.get_current_country")
    def test_initials(self, mock_get_current_country):
        mock_get_current_country.result = "uganda"
        patient_log = self.get_patient_log(
            gender=FEMALE,
            initials="XX",
            willing_to_screen=YES,
            screening_refusal_reason=None,
        )
        cleaned_data = dict(
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=FEMALE,
            initials="ZZ",
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Invalid. Expected XX", str(form._errors.get("initials", "")))

        cleaned_data = dict(
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=FEMALE,
            initials="XX",
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn("initials", form._errors)

    @patch("intecomm_screening.forms.subject_screening.modelform_mixins.get_current_country")
    def test_conditions_matching_patient_log(self, mock_get_current_country):
        mock_get_current_country.result = UGANDA
        patient_log = self.get_patient_log(
            gender=MALE,
            age_in_years=20,
            initials="XX",
            willing_to_screen=YES,
            screening_refusal_reason=None,
            hospital_identifier="123456789",
            conditions=[DM],
        )
        cleaned_data = dict(
            report_datetime=patient_log.report_datetime,
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=MALE,
            age_in_years=20,
            initials="XX",
            hospital_identifier="123456789",
            consent_ability=YES,
            in_care_6m=YES,
            in_care_duration="5y",
            hiv_dx=NO,
            dm_dx=YES,
            htn_dx=YES,
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn("Invalid. Condition not indicated", str(form._errors.get("htn_dx", "")))

        patient_log.conditions.add(Conditions.objects.get(name=HTN))
        patient_log.save()
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertNotIn(
            "Invalid. Condition not indicated", str(form._errors.get("hiv_dx", ""))
        )
        self.assertNotIn(
            "Invalid. Condition not indicated", str(form._errors.get("htn_dx", ""))
        )
        self.assertNotIn(
            "Invalid. Condition not indicated", str(form._errors.get("dm_dx", ""))
        )

        patient_log.conditions.remove(Conditions.objects.get(name=HIV))
        patient_log.conditions.remove(Conditions.objects.get(name=HTN))
        patient_log.conditions.remove(Conditions.objects.get(name=DM))
        patient_log.save()
        cleaned_data.update(hiv_dx=NO, dm_dx=NO, htn_dx=NO)
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn(
            "No conditions (HIV/DM/HTN) have been indicated for this patient",
            str(form._errors.get("__all__", "")),
        )

    @patch("intecomm_screening.forms.subject_screening.modelform_mixins.get_current_country")
    def test_health_talk_reponse_from_patientlog(self, mock_get_current_country):
        mock_get_current_country.result = UGANDA
        patient_log = self.get_patient_log(
            gender=MALE,
            age_in_years=20,
            initials="XX",
            willing_to_screen=YES,
            screening_refusal_reason=None,
            conditions=[DM],
            first_health_talk=TBD,
            second_health_talk=TBD,
        )
        cleaned_data = dict(
            report_datetime=patient_log.report_datetime,
            patient_log_identifier=patient_log.patient_log_identifier,
            gender=MALE,
            age_in_years=20,
            initials="XX",
            consent_ability=YES,
            in_care_6m=YES,
            in_care_duration="5y",
            hiv_dx=NO,
            dm_dx=YES,
            htn_dx=NO,
        )
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn(
            "Has patient attended the first health talk", str(form._errors.get("__all__", ""))
        )

        patient_log.first_health_talk = YES
        patient_log.second_health_talk = TBD
        patient_log.save()
        patient_log.refresh_from_db()

        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()
        self.assertIn(
            "Has patient attended the second health talk", str(form._errors.get("__all__", ""))
        )

        patient_log.first_health_talk = NO
        patient_log.second_health_talk = NO
        patient_log.save()
        patient_log.refresh_from_db()
        form = SubjectScreeningForm(data=cleaned_data, instance=SubjectScreening())
        form.is_valid()

        self.assertNotIn(
            "Has patient attended the first health talk", str(form._errors.get("__all__", ""))
        )
        self.assertNotIn(
            "Has patient attended the second health talk", str(form._errors.get("__all__", ""))
        )
